@@ -14,8 +14,6 @@ import pandas as pd
 import pygit2
 from alpaca_trade_api.entity import Order
 from alpaca_trade_api.rest import APIError
-# from alpaca_trade_api.stream2 import StreamConn
-from google.cloud import error_reporting
 from pandas import DataFrame as df
 from pytz import timezone
 from pytz.tzinfo import DstTzInfo
@@ -28,7 +26,8 @@ from liualgotrader.fincalcs.data_conditions import (QUOTE_SKIP_CONDITIONS,
 from liualgotrader.models.new_trades import NewTrade
 from liualgotrader.models.trending_tickers import TrendingTickers
 from liualgotrader.strategies.base import Strategy, StrategyType
-from liualgotrader.strategies.momentum_long import MomentumLong
+
+shortable: Dict = {}
 
 
 async def end_time(reason: str):
@@ -39,8 +38,71 @@ async def end_time(reason: str):
         )
 
 
+async def is_shortable(trading_api: tradeapi, symbol: str) -> bool:
+
+    asset = None
+    while not asset:
+        try:
+            asset = trading_api.get_asset(symbol)
+        except Exception as e:
+            tlog(f"is_shortable({symbol}) got exception {e}, retrying...")
+            await asyncio.sleep(10)
+
+    return (
+        False
+        if asset.tradable is False
+        or asset.shortable is False
+        or asset.status == "inactive"
+        or asset.easy_to_borrow is False
+        else True
+    )
+
+
+async def liquidator(trading_api: tradeapi) -> None:
+    tlog("liquidator() task starting")
+    try:
+        dt = datetime.today().astimezone(timezone("America/New_York"))
+        to_market_close = (
+            config.market_close - dt
+            if config.market_close > dt
+            else timedelta(hours=24) + (config.market_close - dt)
+        ) - timedelta(minutes=5)
+        await asyncio.sleep(to_market_close.total_seconds())
+
+    except asyncio.CancelledError:
+        tlog("liquidator() cancelled during sleep")
+    except KeyboardInterrupt:
+        tlog("liquidator() - Caught KeyboardInterrupt")
+
+    tlog("liquidator() -> starting liqudation process")
+    try:
+        for symbol in trading_data.positions:
+            tlog(f"liquidator() -> checking {symbol}")
+            if (
+                trading_data.positions[symbol] != 0
+                and trading_data.last_used_strategy[symbol].type
+                == StrategyType.DAY_TRADE
+            ):
+                await liquidate(
+                    symbol, int(trading_data.positions[symbol]), trading_api
+                )
+    except asyncio.CancelledError:
+        tlog("liquidator() cancelled")
+    except KeyboardInterrupt:
+        tlog("liquidator() - Caught KeyboardInterrupt")
+
+    tlog("liquidator() task completed")
+
+
 async def teardown_task(tz: DstTzInfo, task: asyncio.Task) -> None:
     tlog(f"consumer-teardown_task() - starting ")
+
+    if not config.market_close:
+        tlog(
+            "we're probably in market schedule by-pass mode, exiting consumer-teardown_task()"
+        )
+        return
+
     to_market_close: timedelta
     try:
         dt = datetime.today().astimezone(tz)
@@ -85,15 +147,16 @@ async def teardown_task(tz: DstTzInfo, task: asyncio.Task) -> None:
 
 
 async def liquidate(
-    symbol: str, symbol_position: int, trading_api: tradeapi,
+    symbol: str,
+    symbol_position: int,
+    trading_api: tradeapi,
 ) -> None:
 
     if symbol_position:
         tlog(
-            f"Trading over, trying to liquidating remaining position {symbol_position} in {symbol}"
+            f"Trading over, trying to liquidate remaining position {symbol_position} in {symbol}"
         )
         try:
-            trading_data.sell_indicators[symbol] = {"liquidation": 1}
             if symbol_position < 0:
                 o = trading_api.submit_order(
                     symbol=symbol,
@@ -102,9 +165,10 @@ async def liquidate(
                     type="market",
                     time_in_force="day",
                 )
-                op = "buy_short"
-            else:
+                op = "buy"
+                trading_data.buy_indicators[symbol] = {"liquidation": 1}
 
+            else:
                 o = trading_api.submit_order(
                     symbol=symbol,
                     qty=str(symbol_position),
@@ -113,6 +177,7 @@ async def liquidate(
                     time_in_force="day",
                 )
                 op = "sell"
+                trading_data.sell_indicators[symbol] = {"liquidation": 1}
 
             trading_data.open_orders[symbol] = (o, op)
             trading_data.open_order_strategy[
@@ -181,20 +246,19 @@ async def update_partially_filled_order(
     )
 
     try:
-        indicators = (
-            trading_data.buy_indicators[order.symbol]
-            if order.side == "buy"
-            else trading_data.sell_indicators[order.symbol]
-        )
+        indicators = {
+            "buy": trading_data.buy_indicators.get(order.symbol, None),
+            "sell": trading_data.sell_indicators.get(order.symbol, None),
+        }
     except KeyError:
-        indicators = None  # type: ignore
+        indicators = {}
 
     await save(
         order.symbol,
         int(new_qty),
-        trading_data.open_orders.get(order.symbol)[1],  # type: ignore
+        order.side,
         float(order.filled_avg_price),
-        indicators if indicators else "",  # type: ignore
+        indicators,
         order.updated_at,
     )
 
@@ -221,33 +285,29 @@ async def update_filled_order(strategy: Strategy, order: Order) -> None:
     trading_data.positions[order.symbol] += qty
 
     try:
-        indicators = (
-            trading_data.buy_indicators[order.symbol]
-            if order.side == "buy"
-            else trading_data.sell_indicators[order.symbol]
-        )
+        indicators = {
+            "buy": trading_data.buy_indicators.get(order.symbol, None),
+            "sell": trading_data.sell_indicators.get(order.symbol, None),
+        }
     except KeyError:
-        indicators = None  # type: ignore
+        indicators = {}
 
     await save(
         order.symbol,
         int(new_qty),
-        trading_data.open_orders.get(order.symbol)[1],  # type: ignore
+        order.side,
         float(order.filled_avg_price),
-        indicators if indicators else "",  # type: ignore
+        indicators,
         order.filled_at,
     )
 
     if order.side == "buy":
-        trading_data.buy_indicators[order.symbol] = None  # type: ignore
-    else:
-        trading_data.sell_indicators[order.symbol] = None  # type: ignore
-
-    if order.side == "buy":
+        trading_data.buy_indicators.pop(order.symbol, None)
         await strategy.buy_callback(
             order.symbol, float(order.filled_avg_price), int(new_qty)
         )
     else:
+        trading_data.sell_indicators.pop(order.symbol, None)
         await strategy.sell_callback(
             order.symbol, float(order.filled_avg_price), int(new_qty)
         )
@@ -256,10 +316,8 @@ async def update_filled_order(strategy: Strategy, order: Order) -> None:
     trading_data.open_order_strategy.pop(order.symbol, None)
 
 
-async def handle_trade_update(data: Dict) -> bool:
+async def handle_trade_update_for_order(data: Dict) -> bool:
     symbol = data["symbol"]
-    if trading_data.open_orders.get(symbol) is None:
-        return False
 
     last_order = trading_data.open_orders.get(symbol)[0]  # type: ignore
     if last_order is not None:
@@ -281,15 +339,49 @@ async def handle_trade_update(data: Dict) -> bool:
 
         return True
     else:
-        tlog(f"{data['event']} trade update for {symbol} WITHOUT ORDER")
-
+        tlog(
+            f"[ERROR][{data['event']} trade update for {symbol} WITHOUT ORDER, should not arrive here"
+        )
     return False
 
 
-async def handle_data_queue_msg(data: Dict, trading_api: tradeapi) -> bool:
+async def handle_trade_update_wo_order(data: Dict) -> bool:
+    symbol = data["symbol"]
+    event = data["event"]
+    tlog(
+        f"trade update without order for {symbol} data={data} with event {event}"
+    )
+
+    if event == "partial_fill":
+        await update_partially_filled_order(
+            trading_data.last_used_strategy[symbol], Order(data["order"])
+        )
+    elif event == "fill":
+        await update_filled_order(
+            trading_data.last_used_strategy[symbol], Order(data["order"])
+        )
+    elif event in ("canceled", "rejected"):
+        trading_data.partial_fills.pop(symbol, None)
+
+    return True
+
+
+async def handle_trade_update(data: Dict) -> bool:
+    symbol = data["symbol"]
+    if trading_data.open_orders.get(symbol):
+        return await handle_trade_update_for_order(data)
+    else:
+        return await handle_trade_update_wo_order(data)
+
+
+async def handle_data_queue_msg(
+    data: Dict, trading_api: tradeapi, data_api: tradeapi
+) -> bool:
+    global shortable
+
     symbol = data["symbol"]
     if symbol not in market_data.minute_history:
-        _df = trading_api.polygon.historic_agg_v2(
+        _df = data_api.polygon.historic_agg_v2(
             symbol,
             1,
             "minute",
@@ -302,6 +394,9 @@ async def handle_data_queue_msg(data: Dict, trading_api: tradeapi) -> bool:
         tlog(
             f"consumer task loaded {len(market_data.minute_history[symbol].index)} 1-min candles for {symbol}"
         )
+        shortable[symbol] = True  # await is_shortable(data_api, symbol)
+    elif not shortable.get(symbol):
+        shortable[symbol] = True  # await is_shortable(data_api, symbol)
 
     if data["EV"] == "T":
         if "conditions" in data and any(
@@ -365,13 +460,10 @@ async def handle_data_queue_msg(data: Dict, trading_api: tradeapi) -> bool:
         # tlog(f"{symbol} voi:{trading_data.voi[symbol]}")
 
     elif data["EV"] in ("A", "AM"):
-        # First, aggregate 1s bars for up-to-date MACD calculations
         original_ts = ts = pd.Timestamp(
             data["start"], tz="America/New_York", unit="ms"
         )
-        ts = ts.replace(
-            second=0, microsecond=0
-        )  # timedelta(seconds=ts.second, microseconds=ts.microsecond)
+        ts = ts.replace(second=0, microsecond=0)
 
         try:
             current = market_data.minute_history[symbol].loc[ts]
@@ -403,13 +495,20 @@ async def handle_data_queue_msg(data: Dict, trading_api: tradeapi) -> bool:
 
         if data["EV"] == "A":
             if (time_diff := datetime.now(tz=timezone("America/New_York")) - original_ts) > timedelta(seconds=8):  # type: ignore
-                tlog(f"consumer A$ {symbol} out of sync w {time_diff}")
+                tlog(f"A$ {symbol} too out of sync w {time_diff}")
                 return False
+            elif (
+                curr_min := datetime.now(
+                    tz=timezone("America/New_York")
+                ).replace(second=0, microsecond=0)
+            ) > ts:
+                return True
         elif data["EV"] == "AM":
             return True
 
         # Next, check for existing orders for the stock
         existing_order = trading_data.open_orders.get(symbol)
+
         if existing_order is not None:
             existing_order = existing_order[0]
             try:
@@ -456,77 +555,91 @@ async def handle_data_queue_msg(data: Dict, trading_api: tradeapi) -> bool:
         if (
             until_market_close.seconds // 60
             <= config.market_liquidation_end_time_minutes
+            and symbol_position != 0
             and trading_data.last_used_strategy[symbol].type
             == StrategyType.DAY_TRADE
         ):
             await liquidate(symbol, int(symbol_position), trading_api)
+        else:
+            # run strategies
+            for s in trading_data.strategies:
+                if (
+                    "symbol_strategy" in data
+                    and data["symbol_strategy"]
+                    and s.name != data["symbol_strategy"]
+                ):
+                    continue
 
-        # run strategies
-        for s in trading_data.strategies:
-            do, what = await s.run(
-                symbol,
-                int(symbol_position),
-                market_data.minute_history[symbol],
-                ts,
-                trading_api=trading_api,
-            )
+                do, what = await s.run(
+                    symbol,
+                    shortable[symbol],
+                    int(symbol_position),
+                    market_data.minute_history[symbol],
+                    ts,
+                    trading_api=trading_api,
+                    portfolio_value=config.portfolio_value,
+                )
 
-            if do:
-                try:
+                if do:
+                    try:
+                        if what["type"] == "limit":
+                            o = trading_api.submit_order(
+                                symbol=symbol,
+                                qty=what["qty"],
+                                side=what["side"],
+                                type="limit",
+                                time_in_force="day",
+                                limit_price=what["limit_price"],
+                            )
+                        else:
+                            o = trading_api.submit_order(
+                                symbol=symbol,
+                                qty=what["qty"],
+                                side=what["side"],
+                                type=what["type"],
+                                time_in_force="day",
+                            )
 
-                    if what["type"] == "limit":
-                        o = trading_api.submit_order(
-                            symbol=symbol,
-                            qty=what["qty"],
-                            side=what["side"],
-                            type="limit",
-                            time_in_force="day",
-                            limit_price=what["limit_price"],
+                        trading_data.open_orders[symbol] = (o, what["side"])
+                        trading_data.open_order_strategy[symbol] = s
+
+                        tlog(
+                            f"executed strategy {s.name} on {symbol} w data {market_data.minute_history[symbol][-10:]}"
                         )
-                    else:
-                        o = trading_api.submit_order(
-                            symbol=symbol,
-                            qty=what["qty"],
-                            side=what["side"],
-                            type=what["type"],
-                            time_in_force="day",
-                        )
-
-                    trading_data.open_orders[symbol] = (o, what["side"])
-                    trading_data.open_order_strategy[symbol] = s
-
-                    tlog(
-                        f"executed strategy {s.name} on {symbol} w data {market_data.minute_history[symbol][-10:]}"
-                    )
-                    if what["side"] == "buy":
                         trading_data.last_used_strategy[symbol] = s
-                        trading_data.buy_time[symbol] = datetime.now(
-                            tz=timezone("America/New_York")
-                        ).replace(second=0, microsecond=0)
-                        break
-                except APIError as e:
-                    tlog(
-                        f"Exception APIError with {e} from {what}, checking if order filled"
-                    )
+                        if what["side"] == "buy":
+                            trading_data.buy_time[symbol] = datetime.now(
+                                tz=timezone("America/New_York")
+                            ).replace(second=0, microsecond=0)
+                            break
+                    except APIError as e:
+                        tlog(
+                            f"Exception APIError with {e} from {what}, checking if order filled"
+                        )
 
     return True
 
 
-async def queue_consumer(queue: Queue, trading_api: tradeapi,) -> None:
+async def queue_consumer(
+    queue: Queue,
+    trading_api: tradeapi,
+    data_api: tradeapi,
+) -> None:
     tlog("queue_consumer() starting")
 
     try:
         while True:
             try:
                 raw_data = queue.get(timeout=2)
-                # print(raw_data)
                 data = json.loads(raw_data)
-                # print(f"got {data}")
+
                 if data["EV"] == "trade_update":
                     tlog(f"received trade_update: {data}")
                     await handle_trade_update(data)
                 else:
-                    if not await handle_data_queue_msg(data, trading_api):
+                    if not await handle_data_queue_msg(
+                        data, trading_api, data_api
+                    ):
                         while not queue.empty():
                             _ = queue.get()
                         tlog("cleaned queue")
@@ -534,6 +647,16 @@ async def queue_consumer(queue: Queue, trading_api: tradeapi,) -> None:
             except Empty:
                 await asyncio.sleep(0)
                 continue
+            except Exception as e:
+                tlog(
+                    f"Exception in queue_consumer(): exception of type {type(e).__name__} with args {e.args} inside loop"
+                )
+                exc_info = sys.exc_info()
+                lines = traceback.format_exception(*exc_info)
+                for line in lines:
+                    tlog(f"error: {line}")
+                traceback.print_exception(*exc_info)
+                del exc_info
 
     except asyncio.CancelledError:
         tlog("queue_consumer() cancelled ")
@@ -542,6 +665,9 @@ async def queue_consumer(queue: Queue, trading_api: tradeapi,) -> None:
             f"Exception in queue_consumer(): exception of type {type(e).__name__} with args {e.args}"
         )
         exc_info = sys.exc_info()
+        lines = traceback.format_exception(*exc_info)
+        for line in lines:
+            tlog(f"error: {line}")
         traceback.print_exception(*exc_info)
         del exc_info
     finally:
@@ -577,18 +703,27 @@ def get_trading_windows(tz, api):
 
 
 async def consumer_async_main(
-    queue: Queue, symbols: List[str], unique_id: str, strategies_conf: Dict,
+    queue: Queue,
+    symbols: List[str],
+    unique_id: str,
+    strategies_conf: Dict,
 ):
     await create_db_connection(str(config.dsn))
 
-    try:
-        trending_db = TrendingTickers(unique_id)
-        for symbol in symbols:
-            await trending_db.save(symbol)
-    except Exception as e:
-        tlog(
-            f"Exception in consumer_async_main() while storing symbols to DB:{type(e).__name__} with args {e.args}"
-        )
+    if symbols:
+        try:
+            trending_db = TrendingTickers(unique_id)
+            await trending_db.save(symbols)
+        except Exception as e:
+            tlog(
+                f"Exception in consumer_async_main() while storing symbols to DB:{type(e).__name__} with args {e.args}"
+            )
+            exc_info = sys.exc_info()
+            lines = traceback.format_exception(*exc_info)
+            for line in lines:
+                tlog(f"error: {line}")
+            traceback.print_exception(*exc_info)
+            del exc_info
 
     base_url = (
         config.prod_base_url if config.env == "PROD" else config.paper_base_url
@@ -606,46 +741,48 @@ async def consumer_async_main(
     trading_api = tradeapi.REST(
         base_url=base_url, key_id=api_key_id, secret_key=api_secret
     )
+    data_api = tradeapi.REST(
+        base_url=config.prod_base_url,
+        key_id=config.prod_api_key_id,
+        secret_key=config.prod_api_secret,
+    )
     nyc = timezone("America/New_York")
     config.market_open, config.market_close = get_trading_windows(
         nyc, trading_api
     )
     strategy_types = []
-    print("+=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=+")
-    for strategy in strategies_conf:
-        strategy_name = list(strategy.keys())[0]
-        strategy_details = strategy[strategy_name]
-        if strategy_name == "MomentumLong":
-            tlog(f"strategy {strategy_name} selected")
-            strategy_types += [(MomentumLong, strategy_details)]
-        else:
-            tlog(f"custom strategy {strategy_name} selected")
+    for strategy_name in strategies_conf:
+        strategy_details = strategies_conf[strategy_name]
+        tlog(f"custom strategy {strategy_name} selected")
 
-            try:
-                spec = importlib.util.spec_from_file_location(
-                    "module.name", strategy_details["filename"]
-                )
-                custom_strategy_module = importlib.util.module_from_spec(spec)
-                spec.loader.exec_module(  # type: ignore
-                    custom_strategy_module
-                )
-                class_name = list(strategy.keys())[0]
-                custom_strategy = getattr(custom_strategy_module, class_name)
+        try:
+            spec = importlib.util.spec_from_file_location(
+                "module.name", strategy_details["filename"]
+            )
+            custom_strategy_module = importlib.util.module_from_spec(spec)
+            spec.loader.exec_module(custom_strategy_module)  # type: ignore
+            class_name = strategy_name
 
-                if not issubclass(custom_strategy, Strategy):
-                    tlog(
-                        f"custom strartegy must inherit from class {Strategy.__name__}"
-                    )
-                    exit(0)
-                strategy_details.pop("filename", None)
-                strategy_types += [(custom_strategy, strategy_details)]
+            custom_strategy = getattr(custom_strategy_module, class_name)
 
-            except Exception as e:
+            if not issubclass(custom_strategy, Strategy):
                 tlog(
-                    f"[Error]exception of type {type(e).__name__} with args {e.args}"
+                    f"custom strartegy must inherit from class {Strategy.__name__}"
                 )
                 exit(0)
+            strategy_details.pop("filename", None)
+            strategy_types += [(custom_strategy, strategy_details)]
 
+        except FileNotFoundError as e:
+            tlog(f"[Error] file not found `{strategy_details['filename']}`")
+            exit(0)
+        except Exception as e:
+            tlog(
+                f"[Error]exception of type {type(e).__name__} with args {e.args}"
+            )
+            exit(0)
+
+    loaded = 0
     for strategy_tuple in strategy_types:
         strategy_type = strategy_tuple[0]
         strategy_details = strategy_tuple[1]
@@ -654,35 +791,46 @@ async def consumer_async_main(
         await s.create()
 
         trading_data.strategies.append(s)
-        await load_current_long_positions(trading_api, symbols, s)
+        if symbols:
+            loaded += await load_current_positions(trading_api, symbols, s)
+
+    if symbols and loaded != len(symbols):
+        tlog(
+            f"[ERROR] Consumer process loaded only {loaded} out of {len(symbols)} open positions. HINT: make sure that your tradeplan.toml file includes all strategues in previous trading session."
+        )
 
     queue_consumer_task = asyncio.create_task(
-        queue_consumer(queue, trading_api)
+        queue_consumer(queue, trading_api, data_api)
     )
 
+    liquidate_task = asyncio.create_task(liquidator(trading_api))
+
     tear_down = asyncio.create_task(
         teardown_task(timezone("America/New_York"), queue_consumer_task)
     )
     await asyncio.gather(
-        tear_down, queue_consumer_task, return_exceptions=True,
+        tear_down,
+        liquidate_task,
+        queue_consumer_task,
+        return_exceptions=True,
     )
 
     tlog("consumer_async_main() completed")
 
 
-async def load_current_long_positions(
+async def load_current_positions(
     trading_api: tradeapi, symbols: List[str], strategy: Strategy
-) -> None:
+) -> int:
+    loaded = 0
     for symbol in symbols:
         try:
             position = trading_api.get_position(symbol)
-        except Exception:
-            position = None
+        except Exception as e:
+            tlog(f"failed to load open position for {symbol} w/ {e}")
+            continue
 
         if position:
-            tlog(
-                f"loading current position for {symbol} for strategy {strategy.name}"
-            )
+
             try:
                 (
                     prev_run_id,
@@ -691,12 +839,16 @@ async def load_current_long_positions(
                     target_price,
                     indicators,
                     timestamp,
-                ) = await NewTrade.load_latest_long(
+                ) = await NewTrade.load_latest(
                     config.db_conn_pool, symbol, strategy.name
                 )
 
                 if prev_run_id is None:
-                    return
+                    continue
+
+                tlog(
+                    f"loading current position for {symbol} for strategy {strategy.name}"
+                )
 
                 trading_data.positions[symbol] = int(position.qty)
                 trading_data.stop_prices[symbol] = stop_price
@@ -706,11 +858,6 @@ async def load_current_long_positions(
                 ] = trading_data.latest_scalp_basis[symbol] = price
                 trading_data.open_order_strategy[symbol] = strategy
                 trading_data.last_used_strategy[symbol] = strategy
-                trading_data.symbol_resistance[symbol] = (
-                    indicators["resistances"][0]
-                    if "resistances" in indicators
-                    else None
-                )
                 trading_data.buy_time[symbol] = timestamp.astimezone(
                     tz=timezone("America/New_York")
                 )
@@ -722,13 +869,17 @@ async def load_current_long_positions(
                     f"moved {symbol} from {prev_run_id} to {strategy.algo_run.run_id}"
                 )
 
+                loaded += 1
+
             except ValueError:
                 pass
             except Exception as e:
                 tlog(
-                    f"load_current_long_positions() for {symbol} could not load latest trade from db due to exception of type {type(e).__name__} with args {e.args}"
+                    f"load_current_positions() for {symbol} could not load latest trade from db due to exception of type {type(e).__name__} with args {e.args}"
                 )
 
+    return loaded
+
 
 def consumer_main(
     queue: Queue,
@@ -749,6 +900,13 @@ def consumer_main(
         config.build_label = liualgotrader.__version__ if hasattr(liualgotrader, "__version__") else ""  # type: ignore
 
     config.bypass_market_schedule = conf.get("bypass_market_schedule", False)
+    config.portfolio_value = conf.get("portfolio_value", None)
+    if "risk" in conf:
+        config.risk = conf["risk"]
+    if "market_liquidation_end_time_minutes" in conf:
+        config.market_liquidation_end_time_minutes = conf[
+            "market_liquidation_end_time_minutes"
+        ]
 
     market_data.minute_history = minute_history
     try:
