@@ -38,6 +38,7 @@ class Strategy:
     async def run(
         self,
         symbol: str,
+        shortable: bool,
         position: int,
         minute_history: df,
         now: datetime,
@@ -53,8 +54,7 @@ class Strategy:
             True
             if (
                 any(
-                    (now - config.market_open).seconds // 60
-                    >= schedule["start"]
+                    (now - config.market_open).seconds // 60 >= schedule["start"]
                     for schedule in self.schedule
                 )
                 or (
